==> egg-user-service-1.0.0-sources.jar: com/bxm/egg/user/info/impl/UserInfoCacheServiceImpl.java <==
package com.bxm.egg.user.info.impl;

import com.alibaba.fastjson.JSON;
import com.bxm.egg.common.enums.UserStatusEnum;
import com.bxm.egg.user.constant.BaseCacheKeyConfig;
import com.bxm.egg.user.constant.RedisConfig;
import com.bxm.egg.user.equitylevelmedal.UserEquityLevelMedalService;
import com.bxm.egg.user.info.UserInfoCacheService;
import com.bxm.egg.user.info.UserInfoService;
import com.bxm.egg.user.info.UserInformationService;
import com.bxm.egg.user.location.UserLocationService;
import com.bxm.egg.user.model.bo.UserCacheInfoBO;
import com.bxm.egg.user.model.bo.UserInformationBO;
import com.bxm.egg.user.model.bo.UserLocationBO;
import com.bxm.egg.user.model.entity.UserInfoEntity;
import com.bxm.egg.user.model.entity.UserInformationEntity;
import com.bxm.egg.user.model.entity.UserLocationEntity;
import com.bxm.egg.user.model.equitylevelmedal.UserEquityLevelMedalInfoDTO;
import com.bxm.egg.user.properties.HomePageConfigProperties;
import com.bxm.egg.user.properties.UserProperties;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.redis.RedisStringAdapter;
import com.bxm.newidea.component.sync.core.CacheHolder;
import com.bxm.newidea.component.sync.core.SyncCacheHolderFactory;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;
import static org.springframework.util.CollectionUtils.isEmpty;

/**
 * 用户信息缓存service类
 *
 * @author wzy
 * @version 1.0
 * @date 2021/9/17 3:00 下午
 */
@Slf4j
@Service
public class UserInfoCacheServiceImpl implements UserInfoCacheService {

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private UserLocationService userLocationService;

    @Resource
    private RedisStringAdapter redisStringAdapter;

    @Resource
    private UserInformationService userInformationService;

    @Resource
    private UserEquityLevelMedalService userEquityLevelMedalService;

    @Resource
    private UserProperties userProperties;

    @Resource
    private HomePageConfigProperties homePageConfigProperties;

    private CacheHolder getCacheHolder() {
        return SyncCacheHolderFactory.builder()
                .duration(30)
                .timeUnit(TimeUnit.MINUTES)
                .keyGenerator(BaseCacheKeyConfig.USER_INFO_CACHE_KEY)
                .cacheLoader(this::getUserInfoFromRedisOrDB)
                .build();
    }

    @Override
    public UserCacheInfoBO load(Long userId) {
        return getCacheHolder().get(BaseCacheKeyConfig.USER_INFO_CACHE_KEY, userId);
    }

    @Override
    public List<UserCacheInfoBO> getBatchUserInfo(Collection<Long> userIds) {
        List<UserCacheInfoBO> userInfos = Lists.newArrayList();

        if (!isEmpty(userIds)) {
            userIds.forEach(userId -> {
                UserCacheInfoBO userInfoBO = load(userId);
                if (null != userInfoBO) {
                    userInfos.add(userInfoBO);
                }
            });
        }
        return userInfos;
    }

    @Override
    public void clearUserCache(Long userId) {
        //清空二级缓存
        getCacheHolder().evict(BaseCacheKeyConfig.USER_INFO_CACHE_KEY, userId);
        //清空redis缓存
        redisStringAdapter.remove(buildUserKey(userId));
    }

    /**
     * 获取用户信息通过Redis或者DB
     *
     * @param userId 用户id
     * @return 用户信息缓存对象
     */
    public UserCacheInfoBO getUserInfoFromRedisOrDB(Long userId) {
        //先从redis里面拿
        UserCacheInfoBO userCacheInfo = getUserInfoFromRedis(userId);

        if (log.isDebugEnabled()) {
            log.debug("用户信息：{}", JSON.toJSONString(userCacheInfo));
        }

        //用户不存在或者已被注销
        if (null == userCacheInfo || userCacheInfo.getUserId() == -1) {
            return buildInvalidUserCacheInfo(userId);
        }

        return userCacheInfo;
    }


    /**
     * 用户信息查询从Redis缓存中获取
     *
     * @param userId 用户id
     * @return 用户缓存信息
     */
    public UserCacheInfoBO getUserInfoFromRedis(Long userId) {

        UserCacheInfoBO userCacheInfoBO = redisStringAdapter.get(RedisConfig.USER_INFO_CACHE_KEY, UserCacheInfoBO.class);

        //如果存在则直接返回
        if (userCacheInfoBO != null) {
            return userCacheInfoBO;
        }
        //如果用户缓存信息为空，则查询数据库
        userCacheInfoBO = getUserInfoFromDB(userId);

        if (userCacheInfoBO == null) {
            //放一个无效防止缓存穿透
            UserCacheInfoBO invalidUserInfo = new UserCacheInfoBO();
            invalidUserInfo.setUserId(-1L);

            log.warn("加载不存在的用户,用户id:{}", userId);

            redisStringAdapter.set(buildUserKey(userId), invalidUserInfo, userProperties.getUserCacheExpiredSeconds());

            return invalidUserInfo;
        } else {
            redisStringAdapter.set(buildUserKey(userId), userCacheInfoBO, userProperties.getUserCacheExpiredSeconds());
        }

        return userCacheInfoBO;

    }

    /**
     * 获取用户缓存信息从DB
     *
     * @param userId 用户id
     * @return 用户缓存信息
     */
    public UserCacheInfoBO getUserInfoFromDB(Long userId) {

        UserInfoEntity userInfoEntity = userInfoService.selectAllUserById(userId);

        if (userInfoEntity == null) {
            return null;
        }

        UserCacheInfoBO userCacheInfoBO = new UserCacheInfoBO();

        userCacheInfoBO.setUserId(userId);

        fillUserInfo(userCacheInfoBO, userInfoEntity);
        //填充用户资料信息
        fillUserInformationInfo(userCacheInfoBO);
        //填充用户定位信息
        fillUserLocationInfo(userCacheInfoBO);
        //填充用户的勋章信息和等级信息
        fillUserMedalInfoAndLevel(userCacheInfoBO);

        return userCacheInfoBO;

    }

    /**
     * 填充用户的基本信息
     *
     * @param userCacheInfoBO 用户缓存对象
     * @param userInfoEntity  用户基本信息实体类
     * @ 用户缓存信息
     */
    private void fillUserInfo(UserCacheInfoBO userCacheInfoBO, UserInfoEntity userInfoEntity) {
        BeanUtils.copyProperties(userInfoEntity, userCacheInfoBO);

        userCacheInfoBO.setUserId(userInfoEntity.getId());
    }


    /**
     * 填充用户的勋章和等级信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserMedalInfoAndLevel(UserCacheInfoBO userCacheInfoBO) {
        UserEquityLevelMedalInfoDTO equityLevelMedalInfo = userEquityLevelMedalService.getEquityLevelMedalInfo(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (equityLevelMedalInfo == null) {
            return;
        }
        userCacheInfoBO.setWearMedalList(equityLevelMedalInfo.getWearMedalDTOList());
        userCacheInfoBO.setUserWarmLevelDTO(equityLevelMedalInfo.getUserEquityDTO());
    }

    /**
     * 填充用户的定位信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserLocationInfo(UserCacheInfoBO userCacheInfoBO) {
        UserLocationEntity userLocationEntity = userLocationService.getUserLocationByUserId(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (userLocationEntity == null) {
            return;
        }
        UserLocationBO userLocationBO = new UserLocationBO();

        userLocationBO.setLocationCode(userLocationEntity.getLocationCode());
        userLocationBO.setLocationName(userLocationEntity.getLocationName());
        userCacheInfoBO.setUserLocationBO(userLocationBO);
    }

    /**
     * 填充用户的资料信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserInformationInfo(UserCacheInfoBO userCacheInfoBO) {
        UserInformationEntity userInformationEntity = userInformationService.getUserInformationByUserId(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (userInformationEntity == null) {
            return;
        }

        UserInformationBO userInformationBO = new UserInformationBO();
        userInformationBO.setPersonalProfile(userInformationEntity.getPersonalProfile());
        userInformationBO.setBirthday(userInformationEntity.getBirthday());

        //设置默认个人简介
        if (isNotEmpty(userInformationBO.getPersonalProfile())) {
            userInformationBO.setIsDefaultPersonalProfile(Boolean.FALSE);
        } else {
            userInformationBO.setIsDefaultPersonalProfile(Boolean.TRUE);
            userInformationBO.setPersonalProfile(homePageConfigProperties.getDefaultPersonalProfile());
        }

        userCacheInfoBO.setUserInformationBO(userInformationBO);

        if (log.isDebugEnabled()) {
            log.debug("用户资料信息：{}", JSON.toJSONString(userInformationBO));
        }
    }

    /**
     * 构造一个失效的用户缓存信息
     *
     * @param userId 用户id
     * @return 用户信息
     */
    private UserCacheInfoBO buildInvalidUserCacheInfo(Long userId) {
        UserCacheInfoBO userCacheInfoBO = new UserCacheInfoBO();

        userCacheInfoBO.setHeadImg(userProperties.getDefaultHeadImageUrl());
        userCacheInfoBO.setUserId(userId);
        userCacheInfoBO.setNickname("未知用户");
        userCacheInfoBO.setState(UserStatusEnum.DISABLE.getCode());

        return userCacheInfoBO;
    }


    private KeyGenerator buildUserKey(Long userId) {
        return RedisConfig.USER_INFO_CACHE_KEY.copy().appendKey(userId);
    }

}

==> egg-user-service-1.0.0.7-sources.jar: com/bxm/egg/user/info/impl/UserInfoCacheServiceImpl.java <==
package com.bxm.egg.user.info.impl;

import com.alibaba.fastjson.JSON;
import com.bxm.egg.common.enums.UserStatusEnum;
import com.bxm.egg.user.constant.BaseCacheKeyConfig;
import com.bxm.egg.user.constant.RedisConfig;
import com.bxm.egg.user.equitylevelmedal.UserEquityLevelMedalService;
import com.bxm.egg.user.info.UserInfoCacheService;
import com.bxm.egg.user.info.UserInfoService;
import com.bxm.egg.user.info.UserInformationService;
import com.bxm.egg.user.location.UserLocationService;
import com.bxm.egg.user.model.bo.UserCacheInfoBO;
import com.bxm.egg.user.model.bo.UserInformationBO;
import com.bxm.egg.user.model.bo.UserLocationBO;
import com.bxm.egg.user.model.entity.UserInfoEntity;
import com.bxm.egg.user.model.entity.UserInformationEntity;
import com.bxm.egg.user.model.entity.UserLocationEntity;
import com.bxm.egg.user.model.equitylevelmedal.UserEquityLevelMedalInfoDTO;
import com.bxm.egg.user.properties.HomePageConfigProperties;
import com.bxm.egg.user.properties.UserProperties;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.redis.RedisStringAdapter;
import com.bxm.newidea.component.sync.core.CacheHolder;
import com.bxm.newidea.component.sync.core.SyncCacheHolderFactory;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;
import static org.springframework.util.CollectionUtils.isEmpty;

/**
 * 用户信息缓存service类
 *
 * @author wzy
 * @version 1.0
 * @date 2021/9/17 3:00 下午
 */
@Slf4j
@Service
public class UserInfoCacheServiceImpl implements UserInfoCacheService {

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private UserLocationService userLocationService;

    @Resource
    private RedisStringAdapter redisStringAdapter;

    @Resource
    private UserInformationService userInformationService;

    @Resource
    private UserEquityLevelMedalService userEquityLevelMedalService;

    @Resource
    private UserProperties userProperties;

    @Resource
    private HomePageConfigProperties homePageConfigProperties;

    private CacheHolder getCacheHolder() {
        return SyncCacheHolderFactory.builder()
                .duration(30)
                .timeUnit(TimeUnit.MINUTES)
                .keyGenerator(BaseCacheKeyConfig.USER_INFO_CACHE_KEY)
                .cacheLoader(this::getUserInfoFromRedisOrDB)
                .build();
    }

    @Override
    public UserCacheInfoBO load(Long userId) {
        return getCacheHolder().get(BaseCacheKeyConfig.USER_INFO_CACHE_KEY, userId);
    }

    @Override
    public List<UserCacheInfoBO> getBatchUserInfo(Collection<Long> userIds) {
        List<UserCacheInfoBO> userInfos = Lists.newArrayList();

        if (!isEmpty(userIds)) {
            userIds.forEach(userId -> {
                UserCacheInfoBO userInfoBO = load(userId);
                if (null != userInfoBO) {
                    userInfos.add(userInfoBO);
                }
            });
        }
        return userInfos;
    }

    @Override
    public void clearUserCache(Long userId) {
        //清空二级缓存
        getCacheHolder().sendEvictCmd(BaseCacheKeyConfig.USER_INFO_CACHE_KEY, userId);
        //清空redis缓存
        redisStringAdapter.remove(buildUserKey(userId));
    }

    /**
     * 获取用户信息通过Redis或者DB
     *
     * @param userId 用户id
     * @return 用户信息缓存对象
     */
    public UserCacheInfoBO getUserInfoFromRedisOrDB(Long userId) {
        //先从redis里面拿
        UserCacheInfoBO userCacheInfo = getUserInfoFromRedis(userId);

        if (log.isDebugEnabled()) {
            log.debug("用户信息：{}", JSON.toJSONString(userCacheInfo));
        }

        //用户不存在或者已被注销
        if (null == userCacheInfo || userCacheInfo.getUserId() == -1) {
            return buildInvalidUserCacheInfo(userId);
        }

        return userCacheInfo;
    }


    /**
     * 用户信息查询从Redis缓存中获取
     *
     * @param userId 用户id
     * @return 用户缓存信息
     */
    public UserCacheInfoBO getUserInfoFromRedis(Long userId) {

        UserCacheInfoBO userCacheInfoBO = redisStringAdapter.get(RedisConfig.USER_INFO_CACHE_KEY, UserCacheInfoBO.class);

        //如果存在则直接返回
        if (userCacheInfoBO != null) {
            return userCacheInfoBO;
        }
        //如果用户缓存信息为空，则查询数据库
        userCacheInfoBO = getUserInfoFromDB(userId);

        if (userCacheInfoBO == null) {
            //放一个无效防止缓存穿透
            UserCacheInfoBO invalidUserInfo = new UserCacheInfoBO();
            invalidUserInfo.setUserId(-1L);

            log.warn("加载不存在的用户,用户id:{}", userId);

            redisStringAdapter.set(buildUserKey(userId), invalidUserInfo, userProperties.getUserCacheExpiredSeconds());

            return invalidUserInfo;
        } else {
            redisStringAdapter.set(buildUserKey(userId), userCacheInfoBO, userProperties.getUserCacheExpiredSeconds());
        }

        return userCacheInfoBO;

    }

    /**
     * 获取用户缓存信息从DB
     *
     * @param userId 用户id
     * @return 用户缓存信息
     */
    public UserCacheInfoBO getUserInfoFromDB(Long userId) {

        UserInfoEntity userInfoEntity = userInfoService.selectAllUserById(userId);

        if (userInfoEntity == null) {
            return null;
        }

        UserCacheInfoBO userCacheInfoBO = new UserCacheInfoBO();

        userCacheInfoBO.setUserId(userId);

        fillUserInfo(userCacheInfoBO, userInfoEntity);
        //填充用户资料信息
        fillUserInformationInfo(userCacheInfoBO);
        //填充用户定位信息
        fillUserLocationInfo(userCacheInfoBO);
        //填充用户的勋章信息和等级信息
        fillUserMedalInfoAndLevel(userCacheInfoBO);

        return userCacheInfoBO;

    }

    /**
     * 填充用户的基本信息
     *
     * @param userCacheInfoBO 用户缓存对象
     * @param userInfoEntity  用户基本信息实体类
     * @ 用户缓存信息
     */
    private void fillUserInfo(UserCacheInfoBO userCacheInfoBO, UserInfoEntity userInfoEntity) {
        BeanUtils.copyProperties(userInfoEntity, userCacheInfoBO);

        userCacheInfoBO.setUserId(userInfoEntity.getId());
    }


    /**
     * 填充用户的勋章和等级信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserMedalInfoAndLevel(UserCacheInfoBO userCacheInfoBO) {
        UserEquityLevelMedalInfoDTO equityLevelMedalInfo = userEquityLevelMedalService.getEquityLevelMedalInfo(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (equityLevelMedalInfo == null) {
            return;
        }
        userCacheInfoBO.setWearMedalList(equityLevelMedalInfo.getWearMedalDTOList());
        userCacheInfoBO.setUserWarmLevelDTO(equityLevelMedalInfo.getUserEquityDTO());
    }

    /**
     * 填充用户的定位信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserLocationInfo(UserCacheInfoBO userCacheInfoBO) {
        UserLocationEntity userLocationEntity = userLocationService.getUserLocationByUserId(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (userLocationEntity == null) {
            return;
        }
        UserLocationBO userLocationBO = new UserLocationBO();

        userLocationBO.setLocationCode(userLocationEntity.getLocationCode());
        userLocationBO.setLocationName(userLocationEntity.getLocationName());
        userCacheInfoBO.setUserLocationBO(userLocationBO);
    }

    /**
     * 填充用户的资料信息
     *
     * @param userCacheInfoBO 用户缓存对象
     */
    private void fillUserInformationInfo(UserCacheInfoBO userCacheInfoBO) {
        UserInformationEntity userInformationEntity = userInformationService.getUserInformationByUserId(userCacheInfoBO.getUserId());

        //如果不存在则直接返回
        if (userInformationEntity == null) {
            return;
        }

        UserInformationBO userInformationBO = new UserInformationBO();
        userInformationBO.setPersonalProfile(userInformationEntity.getPersonalProfile());
        userInformationBO.setBirthday(userInformationEntity.getBirthday());

        //设置默认个人简介
        if (isNotEmpty(userInformationBO.getPersonalProfile())) {
            userInformationBO.setIsDefaultPersonalProfile(Boolean.FALSE);
        } else {
            userInformationBO.setIsDefaultPersonalProfile(Boolean.TRUE);
            userInformationBO.setPersonalProfile(homePageConfigProperties.getDefaultPersonalProfile());
        }

        userCacheInfoBO.setUserInformationBO(userInformationBO);

        if (log.isDebugEnabled()) {
            log.debug("用户资料信息：{}", JSON.toJSONString(userInformationBO));
        }
    }

    /**
     * 构造一个失效的用户缓存信息
     *
     * @param userId 用户id
     * @return 用户信息
     */
    private UserCacheInfoBO buildInvalidUserCacheInfo(Long userId) {
        UserCacheInfoBO userCacheInfoBO = new UserCacheInfoBO();

        userCacheInfoBO.setHeadImg(userProperties.getDefaultHeadImageUrl());
        userCacheInfoBO.setUserId(userId);
        userCacheInfoBO.setNickname("未知用户");
        userCacheInfoBO.setState(UserStatusEnum.DISABLE.getCode());

        return userCacheInfoBO;
    }


    private KeyGenerator buildUserKey(Long userId) {
        return RedisConfig.USER_INFO_CACHE_KEY.copy().appendKey(userId);
    }

}
